package com.yaoyicloud.service.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.time.LocalDate;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import cn.hutool.core.util.IdUtil;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.config.ReportPathManager;
import com.yaoyicloud.constant.enums.ModuleType;
import com.yaoyicloud.constant.enums.ReportType;
import com.yaoyicloud.entity.ReportGenerationResult;
import com.yaoyicloud.render.InterestConflictsRender;
import com.yaoyicloud.render.ProjectInfoRender;
import com.yaoyicloud.render.PublicRecordRender;

import com.yaoyicloud.render.association.AssociationAttachmentSectionRender;
import com.yaoyicloud.render.foundation.FoundationAttachmentSectionRender;
import com.yaoyicloud.render.foundation.FoundationfinancialInfoRender;
import com.yaoyicloud.render.platform.AntiBriberyRender;
import com.yaoyicloud.render.platform.AttachmentSectionRender;
import com.yaoyicloud.render.platform.BasicInfoRender;
import com.yaoyicloud.render.platform.FinancialInfoRender;
import com.yaoyicloud.render.platform.ServiceProviderInfoRender;
import com.yaoyicloud.render.association.AssociationBasicInfoRender;
import com.yaoyicloud.render.foundation.FoundationBasicInfoRender;
import com.yaoyicloud.render.platform.update.AntiBriberyNewRender;
import com.yaoyicloud.render.platform.update.BasicInfoNewRender;
import com.yaoyicloud.render.platform.update.FinancialInfoNewRender;
import com.yaoyicloud.render.platform.update.PublicRecordNewRender;
import com.yaoyicloud.render.platform.update.TaxNewRender;

import com.yaoyicloud.service.ReportUpdateService;

import com.yaoyicloud.tools.DocxUtil;

import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.stereotype.Service;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

import static com.yaoyicloud.config.SessionInterceptor.SESSION_MAP;

@Slf4j
@Service
@RequiredArgsConstructor
public class ReportUpdateServiceImpl implements ReportUpdateService {

    private final FilerepoProperties filerepoProperties;
    private final ReportPathManager reportPathManager;

    /**
     * 按配置项模块
     * 
     * @param reportType
     * @param data
     * @param templateBytes
     * @param outputBasePath
     * @param relationId
     * @param request
     * @param moduleType
     * @param name
     * @param level
     * @param tenantName
     * @return
     * @throws Exception
     */
    @SuppressWarnings("checkstyle:ReturnCount")
    @Override
    public ReportGenerationResult createPlusVersionCheckReport(
        ReportType reportType, String data, byte[] templateBytes, String outputBasePath, Long relationId,
        HttpServletRequest request, ModuleType moduleType, String name, Integer level, String tenantName)
        throws Exception {

        // 1. 将字节流模版写入临时文件
        File tempTemplateFile = File.createTempFile("template_", ".docx");
        try (FileOutputStream fos = new FileOutputStream(tempTemplateFile)) {
            fos.write(templateBytes);
        }
        Map<String, Object> processedData = new HashMap<>();
        processedData.put("reportDate", LocalDate.now().toString());
        processedData.put("type", reportType.getDesc());
        processedData.put("name", name);
        processedData.put("tenantName", tenantName);
        processedData.put("levelInteger", level);

        String sessionId = SESSION_MAP.get(relationId.toString());

        // 处理指定模块
        String reportPath = processModule(moduleType, data, processedData, templateBytes, sessionId, relationId);
        reportPathManager.addReportPath(sessionId, reportPath);

        String mergedReportPath = null;
        boolean isLastModule = reportPath.contains("attachmentSection");

        if (isLastModule) {
            List<String> reportPaths = reportPathManager.getReportPaths(sessionId);
            XWPFDocument targetDoc;
            try (FileInputStream fis = new FileInputStream(reportPaths.get(0))) {
                targetDoc = new XWPFDocument(fis);
            }
            List<XWPFDocument> sourceDocs = new ArrayList<>();
            for (int i = 1; i < reportPaths.size(); i++) {
                try (FileInputStream fis = new FileInputStream(reportPaths.get(i))) {
                    sourceDocs.add(new XWPFDocument(fis));
                }
            }
            String label = relationId + "_" + IdUtil.fastSimpleUUID();
            mergedReportPath = filerepoProperties.getBasePath() + "/" + label + ".docx";
            // 调用合并方法
            DocxUtil.mergeDocx(targetDoc, sourceDocs, mergedReportPath, relationId);
        }

        return new ReportGenerationResult(reportPath, mergedReportPath, isLastModule);
    }

    /**
     * 处理单个模块的报告生成
     */
    @SuppressWarnings("checkstyle:ParameterNumber")
    private String processModule(ModuleType moduleType, String data, Map<String, Object> processedData,
        byte[] templateBytes, String sessionId, Long relationId) throws Exception {
        String reportPath;
        switch (moduleType) {
            case ANTIBRIBERY:
                reportPath = new AntiBriberyRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case ANTIBRIBERY_NEW:
                reportPath = new AntiBriberyNewRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PLATFORM_COMPANY_BASICINFO:
                reportPath = new BasicInfoRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PLATFORM_COMPANY_ATTACHMENTSECTION:
                reportPath = new AttachmentSectionRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case ASSOCIATION_ATTACHMENTSECTION:
                reportPath = new AssociationAttachmentSectionRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case FOUNDATION_ATTACHMENTSECTION:
                reportPath = new FoundationAttachmentSectionRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case COMMON:
                reportPath = new ServiceProviderInfoRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PLATFORM_COMPANY_FINANCIALINFO:
                reportPath = new FinancialInfoRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PLATFORM_COMPANY_FINANCIALINFO_NEW:
                reportPath = new FinancialInfoNewRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case FOUNDATION_FINANCIALINFO:
                reportPath = new FoundationfinancialInfoRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PUBLIC_RECORD:
                reportPath = new PublicRecordRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PUBLIC_RECORD_NEW:
                reportPath = new PublicRecordNewRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PLATFORM_COMPANY_BASICINFO_NEW:
                reportPath = new BasicInfoNewRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case FOUNDATION_BASICINFO:
                reportPath = new FoundationBasicInfoRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case ASSOCIATION_BASICINFO:
                reportPath = new AssociationBasicInfoRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case PROJECT:
                reportPath = new ProjectInfoRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case INTERESTCONFLICTS:
                reportPath = new InterestConflictsRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            case TAX_NEW:
                reportPath = new TaxNewRender(sessionId, filerepoProperties)
                    .renderDocx(data, processedData, templateBytes, String.valueOf(relationId));
                break;
            default:
                throw new UnsupportedOperationException("Unsupported module type: " + moduleType);
        }
        return reportPath;
    }

}
